/*
 * UCTRAVE.cpp
 *
 *  Created on: Mar 28, 2011
 *      Author: Chang Liu
 *
 *
 * Copyright 2011 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "UCTRAVE.h"
#include <math.h>
#include <time.h>
#include <cfloat>
#include <iostream>

UCTRAVE::UCTRAVE(double effectiveness, double discount, double equivalenceParam, double fpuConst):
	RolloutMonteCarlo(effectiveness, discount),
	explorationConstUCT(0.7),
	explorationConstRAVE(0.7),
	equivalenceParam(equivalenceParam),
	fpuConst(fpuConst)
{}

UCTRAVE::~UCTRAVE()
{}

double UCTRAVE::evaluate(GameState& state){
	double valueUCT;
	double valueRAVE;
	double value; //final linear combination of UCT and RAVE

	double biasUCT = explorationBiasUCT(state);
	double biasRAVE = explorationBiasRAVE(state);

	double parentVisits = state.getParent()->searchData.numVisits;
	double beta = sqrt(equivalenceParam/(3 * parentVisits + equivalenceParam));

	if(state.getPlayer() == B) { //we like low values
		valueUCT = 1 - state.searchData.empiricalValue;
		valueRAVE = 1 - state.searchData.empiricalValueRAVE;
	} else { //we like high values
		valueUCT = state.searchData.empiricalValue;
		valueRAVE = state.searchData.empiricalValueRAVE;
	}

	value = beta * (valueRAVE + biasRAVE) + (1 - beta) * (valueUCT + biasUCT);
	return value;
}

double UCTRAVE::evaluateFirst(GameState& parent){
	return fpuConst;
}

double UCTRAVE::explorationBiasUCT(GameState& state) {
	double logParentVisits = log(state.getParent()->searchData.numVisits);

	double biasUCT = explorationConstUCT * sqrt(logParentVisits/state.searchData.numVisits);

	return biasUCT;
}


double UCTRAVE::explorationBiasRAVE(GameState& state) {
	double logParentVisits = log(state.getParent()->searchData.numVisitsRAVE);

	double biasRAVE = explorationConstRAVE * sqrt(logParentVisits/state.searchData.numVisitsRAVE);

	return biasRAVE;
}

std::string UCTRAVE::toString() const {
	std::stringstream ss;
	ss << "uctrave";
	return ss.str();
}
