/*
 * UCTRAVE.h
 *
 *  Created on: Mar 28, 2011
 *      Author: Chang Liu
 *
 *
 * Copyright 2011 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef UCTRAVE_H_
#define UCTRAVE_H_

#include "RolloutMonteCarlo.h"

/**
 * Represents an instance of the UCT algorithm. UCT recursively treats each internal node
 * of the tree as a separate UCT problem, selecting moves using the UCB1 policy. The reward
 * values from UCB1 for each node are the result of running UCT on internal nodes. UCT quickly
 * converges to a near-optimal move without excessive exploration of the tree, and will find
 * the optimal move eventually as it never stops exploring.
 */
class UCTRAVE : public RolloutMonteCarlo{
public:
	UCTRAVE(double simulationEffectiveness, double propogationConstant, double equivalenceParam, double fpuConst=10000);
	virtual ~UCTRAVE();

	virtual std::string toString() const;

protected:
	double curBest;
	double explorationConstUCT; //k for UCT
	double explorationConstRAVE;//k for RAVE
	double equivalenceParam;
	double fpuConst;

	/**evaluate a move combined the value of UCT and RAVE
	 *
	 */
	virtual double evaluate(GameState& root);
	/**evaluate first
	 *
	 */
	virtual double evaluateFirst(GameState& parent);

	/**calculate the bias for RAVE
	 *
	 */
	virtual double explorationBiasRAVE(GameState& state);
	/**calucate the bias for regular UCT
	 *
	 */
	virtual double explorationBiasUCT(GameState& state);
};

#endif /* UCTRAVE_H_ */
